@@ -1,10 +1,11 @@
 #include <iostream>
 
-int main(int argc, char *argv[]) {
+using namespace std;
 
+int main(int argc, char *argv[]) {
   /* An annoying "Hello World" example */
   for (auto i = 0; i < 0xFFFF; i++)
-    cout << "Hello, World!" << endl;
+    cout << "Heljkljlkjlkjlkjkljlhgflo, World!" << endl;
 
   char c = '\n';
   unordered_map<string, vector<string>> m;
